--[[
	StopFullCombine.lua
	
	Autor: 		Ifko[nator]
	Datum: 		31.03.2022
	Version: 	1.0
	
	Changelog:	v1.0 @31.03.2022 - initial implementation in FS 22
]]

StopFullCombine = {};

function StopFullCombine.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Combine, specializations);
end;

function StopFullCombine.registerFunctions(vehicleType)
	local newFunctions = {
		"lockCombine",
		"resetTimer",
		"startBeaconLights",
		"stopBeaconLights"
	};
	
	for _, newFunction in ipairs(newFunctions) do
		SpecializationUtil.registerFunction(vehicleType, newFunction, StopFullCombine[newFunction]);
	end;
end;

function StopFullCombine.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, StopFullCombine);
	end;
end;

function StopFullCombine:onLoad(savegame)
	local specCombine = self.spec_combine;
	
	self:resetTimer(specCombine);

	specCombine.stopTimerMax = 5000; --## stop combine for 5 seconds
	specCombine.needTurnOnBeaconLights = true;
	specCombine.beaconLightsWasTurnedOn = false;
end;

function StopFullCombine:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if self:getIsActiveForInput(true, false) then
		local specCombine = self.spec_combine;
		local specLights = self.spec_lights;
		
		if specCombine ~= nil and specLights ~= nil and #specLights.beaconLights > 0 and self.getIsMotorStarted ~= nil and self:getIsMotorStarted() then
			if self:getFillUnitFillLevelPercentage(specCombine.fillUnitIndex) >= 0.9 then
				self:startBeaconLights(specCombine.needTurnOnBeaconLights, specCombine);
			else
				self:stopBeaconLights(specCombine.beaconLightsWasTurnedOn, specCombine);
			end;
		end;

		if not specCombine.isBufferCombine and self:getFillUnitFreeCapacity(specCombine.fillUnitIndex) == 0 then
			if specCombine.stopTimer < specCombine.stopTimerMax then
				self:lockCombine(self.spec_wheels, self.spec_pipe);

				specCombine.stopTimer = specCombine.stopTimer + dt;
			end;
		else
			if specCombine.stopTimer > 0 then
				self:resetTimer(specCombine);
			end;
		end;
	end;
end;

function StopFullCombine:resetTimer(specCombine)
	specCombine.stopTimer = 0;
end;

function StopFullCombine:lockCombine(specWheels, specPipe)
	if self.isServer then
		for _, wheel in pairs(specWheels.wheels) do
			setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
		end;
	end;

	self:setBrakeLightsVisibility(true);
	self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);

	if specPipe ~= nil and specPipe.hasMovablePipe then
		if specPipe.targetState == 1 then
			self:setPipeState(next(specPipe.unloadingStates));
		end;
	end;
end;

function StopFullCombine:startBeaconLights(needTurnOnBeaconLights, specCombine)
	if needTurnOnBeaconLights then
		self:setBeaconLightsVisibility(true, true, nil);

		specCombine.needTurnOnBeaconLights = false;
		specCombine.beaconLightsWasTurnedOn = true;
	end;
end;

function StopFullCombine:stopBeaconLights(beaconLightsWasTurnedOn, specCombine)
	if beaconLightsWasTurnedOn then
		self:setBeaconLightsVisibility(false, true, nil);

		specCombine.needTurnOnBeaconLights = true;
		specCombine.beaconLightsWasTurnedOn = false;
	end;
end;